@@ -3,14 +3,14 @@ use semver::Version;
 use git2::string_array::StringArray;
 use regex::Regex;
 
-pub struct GitVersion {
-  pub dir: String,
-  pub prefix: String,
+pub struct GitVersion<'a> {
+  pub dir: &'a str,
+  pub prefix: &'a str,
   version: String,
 }
 
-impl GitVersion {
-  pub fn new(dir: String, prefix: String) -> GitVersion {
+impl GitVersion<'_> {
+  pub fn new<'a>(dir: &'a str, prefix: &'a str) -> GitVersion<'a> {
     GitVersion {
       dir,
       prefix,
@@ -19,39 +19,39 @@ impl GitVersion {
   }
 
   pub fn current_version(&mut self) -> String {
-    let repo = match Repository::open(self.dir.clone()) {
-      Ok(repo) => repo,
-      Err(e) => panic!("failed to open: {}", e),
-    };
+    let repo = Repository::open(self.dir)
+        .map_err(|e| format!("Failed to open: {}", e))
+        .unwrap();
 
     let all_tags = self.git_tag(&repo).unwrap();
 
     // filter by a specific regexp
     let pattern = format!("{}{}{}", '^', self.prefix, "\\d+\\.\\d+\\.\\d+$");
-
     let tag_regex = Regex::new(pattern.as_ref()).unwrap();
 
     // sort git versions
-    let mut version_tags: Vec<String> = all_tags.iter().filter(|&tag| {
-      tag_regex.is_match(tag.unwrap())
-    }).map(|tag| tag.unwrap().to_owned()).collect();
+    let mut version_tags: Vec<&str> = all_tags
+        .iter()
+        .filter(|tag| tag_regex.is_match(tag.unwrap()))
+        .map(|tag| tag.unwrap())
+        .collect();
     version_tags.sort_by(|a, b| natord::compare(&a, &b));
 
     // find the greatest version
-    let tag_list_length = version_tags.len().clone();
+    let tag_list_length = version_tags.len();
 
     if tag_list_length != 0 {
-      self.version = version_tags[tag_list_length - 1].clone();
+      self.version = String::from(version_tags[tag_list_length - 1]);
     }
 
     // version
     self.version.to_string()
   }
 
-  pub fn next_version(&mut self, _type: String) -> String {
-    let mut version = Version::parse(self.current_version().as_str()).unwrap();
+  pub fn next_version(&mut self, typ: &str) -> String {
+    let mut version = Version::parse(&self.current_version()).unwrap();
 
-    match _type.as_str() {
+    match typ {
       "major" => version.increment_major(),
       "minor" => version.increment_minor(),
       "patch" => version.increment_patch(),
